// Copyright 2020 RnD Center "ELVEES", JSC

#include <cstring>
#include <iostream>

#include "dmabuf-importer.hpp"

DmaBuf::DmaBuf(size_t size) {
    open_device();

    m_buf.type = ELCORE50_NONCACHED_BUFFER;
    m_buf.size = m_size = size;
    int ret = ioctl(m_elcore_fd, ELCORE50_IOC_CREATE_BUFFER, &m_buf);
    if (ret == -1) throw std::runtime_error("can't create dmabuf");

    m_data = mmap(NULL, size, PROT_READ | PROT_WRITE, MAP_SHARED, m_buf.dmabuf_fd, 0);
    if (m_data == MAP_FAILED) throw std::runtime_error("can't mmap dmabuf");
}

DmaBuf::~DmaBuf() {
    if (m_data) munmap(m_data, m_size);
    if (m_buf.dmabuf_fd > 0) {
        if (close(m_buf.dmabuf_fd)) {
            std::cerr << "Failed to close dmabuf fd" << std::endl;
            abort();
        }
    }
    if (m_elcore_fd > 0) {
        if (close(m_elcore_fd)) {
            std::cerr << "Failed to close elcore fd" << std::endl;
            abort();
        }
    }
}

void DmaBuf::open_device() {
    m_elcore_fd = open("/dev/elcore0", O_RDWR);
    if (m_elcore_fd == -1) throw std::runtime_error("can't open device");
}
